/*
 * map_module.h - header file for westside interface to map client applications.
 *
 * Copyright (C) 2021 IOPSYS Software Solutions AB. All rights reserved.
 *
 * See LICENSE file for license related information.
 *
 */

#ifndef MAP_MODULE_H
#define MAP_MODULE_H


typedef uint32_t mapmodule_object_t;
typedef uint8_t mapmodule_cmdu_mask_t[16];

struct map_module {
	mapmodule_object_t id;
	mapmodule_cmdu_mask_t cmdu_mask;
	char process_cmdu_funcname[64];
};


int map_prepare_cmdu_mask(uint8_t mask[], ...);
const char *map_cmdu_type2str(uint16_t type);
const char *map_tlv_type2str(uint8_t type);

#define map_cmdu_mask_isset(m, f) \
({ \
	(!!(m[(f >= 0x8000 ? 4 : 0) + (f >= 0x8000 ? f - 0x8000 : f) / 8] & \
	(1 << ((f >= 0x8000 ? f - 0x8000 : f) % 8)))); \
})


int map_subscribe(void *bus, void *publisher,
		  const char *name, mapmodule_cmdu_mask_t *mask, void *priv,
		  int (*sub_cb)(void *bus, void *priv, void *data),
		  int (*del_cb)(void *bus, void *priv, void *data),
		  void **subscriber);

int map_unsubscribe(void *bus, void *subscriber);

#endif /* MAP_MODULE_H */
